//+------------------------------------------------------------------+
//|                                            #_i_SpecktrAnalis.mq4 |
//|                                          Copyright  2006, klot. |
//|                                                     klot@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2006, klot."
#property link      "klot@mail.ru"

/*
In signal processing, the cross-correlation (or sometimes "cross-covariance") is a measure 
of similarity of two signals, commonly used to find features in an unknown signal by comparing 
it to a known one. It is a function of the relative time between the signals, is sometimes called 
the sliding dot product, and has applications in pattern recognition and cryptanalysis.
*/

#import "#_lib_FFT.ex4"
void realfastfouriertransform(double& a[], int tnn, bool inversefft);
void fastcorellation(double& signal[], int signallen,  double& pattern[],  int patternlen);
void tworealffts(double a1[], double a2[],  double& a[],  double& b[], int tn); 
void fastconvolution(double& signal[], int signallen, double& response[], int negativelen, int positivelen);   
#import

#property indicator_separate_window
#property indicator_buffers 4
#property indicator_color1 Gold
#property indicator_color2 Aqua
#property indicator_color3 Blue
#property indicator_color4 Lime
#property indicator_level1 0.0
//#property indicator_level1 0.0
//#property indicator_level1 0.0
//---- buffers
double Buffer1[];
double Buffer2[];
double Buffer3[];
double Buffer4[];

//---
extern int n=7; //  
extern int Fmin=2;//2; //    .
extern int Fmax=30;//128; //    .
extern string sym1="Gold";
extern string sym2="DOWJONES";
extern int MaPeriod=6;
extern int pearson_per=5;
double aa[];
double bb[];
double cc[];
double dd[];
int N;

//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
  
   if(StringFind(Symbol(),"m",0)>0)
   {
      sym1=sym1+"m";
      sym2=sym2+"m";
   }   
        
//---- indicators
SetIndexStyle(0,DRAW_LINE,0,2);
SetIndexBuffer(0,Buffer1);
SetIndexEmptyValue(0,0);
SetIndexStyle(1,DRAW_LINE,0,1);
SetIndexBuffer(1,Buffer2);
SetIndexEmptyValue(1,0);
SetIndexStyle(2,DRAW_LINE,0,1);
SetIndexBuffer(2,Buffer3);
SetIndexEmptyValue(2,0);
SetIndexStyle(3,DRAW_LINE,2,1);
SetIndexBuffer(3,Buffer4);
SetIndexEmptyValue(3,0);


//---
   N=MathPow(2,n);
   ArrayResize(aa,N);
   ArrayResize(bb,N);
   ArrayResize(cc,N);
   ArrayResize(dd,N);

if( Fmax>=N ) Fmax=N;
if( Fmin<=2 ) Fmin=2;

//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int start()
  {
   RefreshRates();
   if( Bars<N) return(0);
   
   //---
   for(int i=N-1; i>=0; i--) 
   { 
     aa[i]=iClose(sym1,0,i);
     bb[i]=iClose(sym2,0,i);
     cc[i]=iClose(sym1,0,i);
     dd[i]=iClose(sym2,0,i); 
   }
  
   InSigNormalize(aa);
   InSigNormalize(bb);
   InSigNormalize(cc);
   InSigNormalize(dd);
   realfastfouriertransform(aa, N, false);
   realfastfouriertransform(bb, N, false);
   realfastfouriertransform(cc, N, false);
   realfastfouriertransform(dd, N, false);
   
   
   N=ArraySize(aa);
   //Comment("n= ",N);
   //---
   for( i=0; i<=N-1; i++) 
   { 
      if( i<Fmin || i>Fmax) 
      { 
        aa[i]=0.0;
        bb[i]=0.0;
        cc[i]=0.0;
        dd[i]=0.0; 
      }
   }
   //---
   realfastfouriertransform(aa, N, true);
   realfastfouriertransform(bb, N, true);  
   //realfastfouriertransform(cc, N, true);
   //   fastcorellation(cc,N,dd,N);
   //fastconvolution(cc, N, dd, N, N);   
   //tworealffts(aa, bb,  cc,  dd, N); 

   
   //---
   SetIndexDrawBegin(0,Bars-N);
   SetIndexDrawBegin(1,Bars-N);
   SetIndexDrawBegin(2,Bars-N);
   SetIndexDrawBegin(3,Bars-N);
 
   //----
   double x1[20];
   double y1[20];
   for( i=0; i<=N-1; i++)
   {
      Buffer1[i]=aa[i];
      Buffer2[i]=bb[i];
      
      for (int j=0;j<pearson_per;j++)
      {
        x1[j]= Buffer1[i+j];
        y1[j]= Buffer2[i+j];
      }  
      Buffer3[i]=pearson( x1,y1)/1000;
      //Buffer4[i]=CorrelationIND(Buffer2,Buffer3,20,i)/100;
   }
   
   return(0);
  }
//-------------------------------------------------------------------+

double pearson( double x[], double y[])
{
  int i,j;
  double sxy,sx,sx2,sy,sy2,a,b,c,r;
  int n1 = ArraySize(x);
  for(i=0; i<n1; i++)
  {
    sxy+=x[i]*y[i];
    sx+=x[i];
    sx2+=x[i]*x[i];
    sy+=y[i];
    sy2+=y[i]*y[i];   
  }
  a=sxy-sx*sy/n1;
  b=sx2-sx*sx/n1;
  c=sy2-sy*sy/n1;
  r=a/MathSqrt(b*c);
  return(r);
}

//--------------------------------------------------------------------+
void InSigNormalize(double& aa[])
{
   double sum_sqrt;
   int element_count=ArraySize(aa);
   sum_sqrt=0;
   for( int i=0; i<=element_count-1; i++)
   {
      sum_sqrt+=MathPow(aa[i],2);
   }
   sum_sqrt=MathSqrt(sum_sqrt);
   
   if (sum_sqrt!=0)
   {
      for( i=0; i<=element_count-1; i++)
      {
         aa[i]=aa[i]/sum_sqrt;
      }
   }
   return;
}

